/************************************************************************
 control3.c
 M. Huq
 Set of routines written to evaluate the max of two risky assets based on
 Stulz 1982 (Journal of Financial Economics, vol 10, 161-185 (1982)).
 *************************************************************************/
 #include <stdio.h>
 #include <math.h>
 #include "blackScholes.h"

 /* 
   Cumulative bivariate normal distribution
   N_2(alpha, beta, theta)
   upper limits of integration alpha and beta and correlation theta.

   WHAT DO WE DO HERE? Need case where theta is non-zero. Hence cannot
   do trivial splitting. Here then, we have to diagonalize the covariance
   matrix and then use the product of the Cumulative normal dist in each
   direction. Have to be careful here when we form the matrix based on the
   expressions given in the paper.
 */

/************************************************************************
 In the following we use the following notation (from paper)

 M := European call option price on the min of two underlying stocks.
 MX:= European call option price on the max of two underlying stocks.
 V := Stock price of underlier one
 H := Stock price of underlier two
 rhovh:= correlation between V and H
 sigmav:= volatility of V
 sigmah:= volatility of H
 R := riskfree interest rate
 tau := T-t
 F := Exercise price
 delta := dividend rate. But note this is not taken into account for
          valuation. What is the correct modification here? <--question for
          Scott and Scott!

 *************************************************************************/
double stulzMax(double V, double H, double sigmav, double sigmah, \
          double rhovh, double R, double F, double tau,double delta)
{
  double M, MX, alpha1, alpha2, beta1, beta2, gamma1, gamma2, sigma, rhoc;

  /* Define secondary variables */
  sigma = sqrt(sigmav*sigmav + sigmah*sigmah - 2.0 * rhovh*sigmav*sigmah);

  rhoc = (rhovh * sigmav - sigmah) / sigma;

  gamma1 = (log(H/F) + (R-0.5*sigmah*sigmah)*tau)/(sigma * sqrt(tau));
  gamma2 = (log(V/F) + (R-0.5*sigmav*sigmav)*tau)/(sigma * sqrt(tau));

  alpha1 = gamma1 + sigmah * sqrt(tau);
  beta1  = gamma2 + sigmav * sqrt(tau);

  alpha2 = (log(V/H) - 0.5 * sigma*sigma*sqrt(tau))/(sigma * sqrt(tau));
  beta2  = (log(H/V) - 0.5 * sigma*sigma*sqrt(tau))/(sigma * sqrt(tau));

  /* Evaluate the European call on the min of two risky stocks using
     equation 11. See form of the same page 166 (first eqn in section 3).
  */ 
  M = H * BivarCND(alpha1, alpha2, rhoc) \
    + V * BivarCND(beta1, beta2, rhoc)   \
    - F * exp(-R * tau) * BivarCND(gamma1,gamma2, rhovh);

  /* Now with M compute the European call option on the max of two
     underliers using now equation 12 from Stulz. Note that this formula
     required the pricing of European call options on single assets. For
     this I used the Black-Scholes formula already in the code. */
  MX = euroCall( V,  F,  R,  sigmav,  tau,  delta) \
     + euroCall( H,  F,  R,  sigmav,  tau,  delta) \
     - M;

  return(MX);
} /* End stulzMax */

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* The following is from  a formula in Kolb, "Futures, Options and Swaps"
   An independent description of what is in Stulz above. 
   Gives the same answer as Kolb page 603. Verified.
*/
double maxOf2Risk(double S1, double S2, double X, double tau, \
       double sigma1, double sigma2, double r, double delta1, \
       double delta2, double rho){
  double Sigma, rho1, rho2, w1, w2, w3, w4, Q1, Q2, Q3;

  Sigma = sqrt(sigma1*sigma1 + sigma2*sigma2 - 2.0 * rho*sigma1*sigma2);
  rho1 = (rho * sigma2 - sigma1)/Sigma;
  rho2 = (rho * sigma1 - sigma2)/Sigma;

  w1 = log(S1/X) + (r - delta1 + 0.5 * sigma1*sigma1)*tau;
  w1 /= sigma1 * sqrt(tau);

  w2 = log(S2/X) + (r - delta2 + 0.5 * sigma2*sigma2)*tau;
  w2 /= sigma2 * sqrt(tau);

  w3 = log(S1/S2) + (delta2 - delta1 + 0.5 * Sigma*Sigma)*tau;
  w3 /= Sigma * sqrt(tau);

  w4 = log(S2/S1) + (delta1 - delta2 + 0.5 * Sigma*Sigma)*tau;
  w4 /= Sigma * sqrt(tau);

  Q1 = S1 * exp(-delta1*tau) * \
      (cumNorm(w3) - BivarCND(-w1,w3,rho1));
  Q2 = S2 * exp(-delta2*tau) * \
      (cumNorm(w4) - BivarCND(-w2,w4,rho2));
  Q3 = X * exp(-r*tau) * \
      BivarCND(-w1 + sigma1*sqrt(tau), -w2+sigma2*sqrt(tau), rho);

  return(Q1 + Q2 + Q3 - X * exp(-r*tau));    
}
