/* $Id: util.c,v 2.3 1998/04/19 23:06:18 mijan Exp $ */
#include <stdio.h>
#include <stdlib.h>
#include "global.h"
#include "functions.h"
#include "util.h"
#include "ranlib.h"
#include "our_getopt.h"
#include "plist.h"

/* Functions to make memory allocation more readable.
 *
 * It is not hard to make these routines give arbitrary index ranges
 * (ala Numerical Recipes) but the arrays in this code naturally start
 * at 0 anyway.
 */

void allocate1dPLIST(PLIST **array, int size){
  int i;

  (*array) = (PLIST*)calloc(size,sizeof(PLIST));
  for(i=0; i<size; i++){
    plist_initialize( &((*array)[i]) );
  } 
}

void deallocate1dPLIST(PLIST **array, int size){
  int i;

  for(i=0; i<size; i++){
    plist_finalize( &((*array)[i]) );
  }
  free(array);
}


void allocate1dDouble(double **array, int size){
  /* Allocates memory for a one dimensional array of doubles with dimensions
   * [0:size-1].  This is really trivial.  Its just here so that 1d
   * allocations look like 2d and 3d allocations.
   */

  (*array) = (double*)calloc(size,sizeof(double));
}

void deallocate1dDouble(double **array, int size){
  /* Allocates memory for a one dimensional array of doubles with dimensions
   * [0:size-1].  This is really trivial.  Its just here so that 1d
   * allocations look like 2d and 3d allocations.
   */

  free(*array);
}


void allocate2dDouble(double ***array, int size1, int size2){
  /* Allocates memory for a 2d array of doubles with dimensions
   * [0:size1-1][0:size2-1].
   */

  int i;

  (*array) = (double**)calloc(size1,sizeof(double*));
  for(i=0; i<size1; i++){
    (*array)[i] = (double*)calloc(size2,sizeof(double));
  }
}


void deallocate2dDouble(double ***array, int size1, int size2){
  /* Deallocates memory for a 2d array of doubles with dimensions
   * [0:size1-1][0:size2-1].
   */

  int i;

  for(i=0; i<size1; i++){
    free((*array)[i]);
  }
  free(*array);
}


void allocate3dDouble(double ****array, int size1, int size2, int size3){
  /* Allocates memory for a 3d array of doubles with dimensions
   * [0:size1-1][0:size2-1][0:size3-1].
   */

  int i,j;

  (*array) = (double***)calloc(size1,sizeof(double**));
  for(i=0; i<size1; i++){
    (*array)[i] = (double**)calloc(size2,sizeof(double*));
    for(j=0; j<size2; j++){
      (*array)[i][j] = (double*)calloc(size3,sizeof(double));
    }
  }
}

void deallocate3dDouble(double ****array, int size1, int size2, int size3){
  /* Dellocates memory for a 3d array of doubles with dimensions
   * [0:size1-1][0:size2-1][0:size3-1].
   */

  int i,j;

  for(i=0; i<size1; i++){
    for(j=0; j<size2; j++){
      free((*array)[i][j]);
    }
    free((*array)[i]);
  }
  free(*array);
}


void allocate1dInt(int **array, int size){
  /* Allocates memory for a one dimensional array of ints with dimensions
   * [0:size-1].  This is really trivial.  Its just here so that 1d
   * allocations look like 2d and 3d allocations.
   */

  (*array) = (int*)calloc(size,sizeof(int));
}

void deallocate1dInt(int **array, int size){
  /* Deallocates memory for a one dimensional array of ints with dimensions
   * [0:size-1].  This is really trivial.  Its just here so that 1d
   * allocations look like 2d and 3d allocations.
   */

  free(*array);
}


void allocate2dInt(int ***array, int size1, int size2){
  /* Allocates memory for a 2d array of ints with dimensions
   * [0:size1-1][0:size2-1].
   */

  int i;

  (*array) = (int**)calloc(size1,sizeof(int*));
  for(i=0; i<size1; i++){
    (*array)[i] = (int*)calloc(size2,sizeof(int));
  }
}


void deallocate2dInt(int ***array, int size1, int size2){
  /* Deallocates memory for a 2d array of int with dimensions
   * [0:size1-1][0:size2-1].
   */

  int i;

  for(i=0; i<size1; i++){
    free((*array)[i]);
  }
  free(*array);
}

void allocate3dInt(int ****array, int size1, int size2, int size3){
  /* Allocates memory for a 3d array of ints with dimensions
   * [0:size1-1][0:size2-1][0:size3-1].
   */

  int i,j;

  (*array) = (int***)calloc(size1,sizeof(int**));
  for(i=0; i<size1; i++){
    (*array)[i] = (int**)calloc(size2,sizeof(int*));
    for(j=0; j<size2; j++){
      (*array)[i][j] = (int*)calloc(size3,sizeof(int));
    }
  }
}


void deallocate3dInt(int ****array, int size1, int size2, int size3){
  /* Dellocates memory for a 3d array of int with dimensions
   * [0:size1-1][0:size2-1][0:size3-1].
   */

  int i,j;

  for(i=0; i<size1; i++){
    for(j=0; j<size2; j++){
      free((*array)[i][j]);
    }
    free((*array)[i]);
  }
  free(*array);
}



void allocate1dFloat(float **array, int size){
  /* Allocates memory for a one dimensional array of floats with dimensions
   * [0:size-1].  This is really trivial.  Its just here so that 1d
   * allocations look like 2d and 3d allocations.
   */

  (*array) = (float*)calloc(size,sizeof(float));
}

void deallocate1dFloat(float **array, int size){
  /* Deallocates memory for a one dimensional array of floats with dimensions
   * [0:size-1].  This is really trivial.  Its just here so that 1d
   * allocations look like 2d and 3d allocations.
   */

  free(*array);
}


void allocate2dFloat(float ***array, int size1, int size2){
  /* Allocates memory for a 2d array of floats with dimensions
   * [0:size1-1][0:size2-1].
   */

  int i;

  (*array) = (float**)calloc(size1,sizeof(float*));
  for(i=0; i<size1; i++){
    (*array)[i] = (float*)calloc(size2,sizeof(float));
  }
}

void allocate3dFloat(float ****array, int size1, int size2, int size3){
  /* Allocates memory for a 3d array of floats with dimensions
   * [0:size1-1][0:size2-1][0:size3-1].
   */

  int i,j;

  (*array) = (float***)calloc(size1,sizeof(float**));
  for(i=0; i<size1; i++){
    (*array)[i] = (float**)calloc(size2,sizeof(float*));
    for(j=0; j<size2; j++){
      (*array)[i][j] = (float*)calloc(size3,sizeof(float));
    }
  }
}

/* Routines for comparing numbers and arrays */

double dMax(double a, double b){
  /* Returns the maximum of a and b */

  return( (a>b) ? a : b);
}

double dMin(double a, double b){
  /* Returns the minimum of a and b */

  return( (a>b) ? b : a);
}

double dArrayMax(int size, double *array){
  /* Returns the largest value in the array.
   * Array has dimensions [0:size-1]
   */

  int i;
  double max;

  max = array[0];
  for(i=1; i<size; i++){
    if(array[i] > max) max = array[i];
  }

  return(max);
}

double dArrayMin(int size, double *array){
  /* Returns the smalles value in the array.
   * Array has dimensions [0:size-1].
   */

  int i;
  double min;

  min = array[0];
  for(i=1; i<size; i++){
    if(array[i] < min) min = array[i];
  }

  return(min);
}

int dWhichMin(int size, double *array){
  /* Returns the position in array[] which contains the smallest value.
   */

  int i, minI;

  minI = 0;
  for(i=1;i<size;i++){
    if(array[i] < array[minI]) minI = i;
  }

  return(minI);
}

int dWhichMax(int size, double *array){
  /* Returns the position in array[] which contains the largets value.
   */

  int i, maxI;

  maxI = 0;
  for(i=1;i<size;i++){
    if(array[i] > array[maxI]) maxI = i;
  }

  return(maxI);
}
